﻿-- Synchronizing module

-- spinning tiles fading in/out times
local SPINNING_TILE_FADE_IN_TIME = 0.07
local SPINNING_TILE_FADE_OUT_TIME = 0.37
local SPINNING_TILE_SIZE = 27

-- client syncing state
-- careful about changing 0, it is hardcoded in the main module
local SYNCING_STATE_NONE = 0
local SYNCING_STATE_CHECK_VERSION = 1
BMTSyncingState = SYNCING_STATE_NONE
-- syncing state texts (for spinning inset in the main frame)
local SYNCING_STATE_CHECK_VERSION_TEXT = "Synchronizing version"

-- timeouts (in seconds)
local SYNCING_TIMEOUT_VERSION = 2

-- addon message prefix
local ADDON_PREFIX = "BMT"
-- addon message separator
local SEPARATOR = "%"

-- syncing keys (strings) sent through addon message
-- version request (also includes this version)
local KEY_VERREQ = "VERREQ"
-- version response
local KEY_VERRESP = "VERRESP";

-- highest revision received from others upon logging in
BMTHighestRevision = BMTRevision;

-- list of people that have the addon - this can be used to ignore people with really old addon versions
-- -- includes:
-- -- name of the player
-- -- channels through which the player communicated with us when syncing version
-- -- version of the addon for that player
BMTPlayersWithAddon = {};

-- if not sending through whisper, target can be nil
function BMTSyncing_SendAddonMessage(msg, channel, target)
	-- just in case
	local channelName = string.upper(channel)
	-- TODO: possibly check if online on whisper
	SendAddonMessage(ADDON_PREFIX, msg, channelName, target)
end

function BMTSyncing_AddonMsgReceiveHandler(prefix, msg, chan, sender)
	if(prefix ~= "BMT") then return end
	BMT_Debug("("..sender..") msg="..msg)
	if(sender == UnitName("player")) then return end
	local channel = string.upper(chan)
	local key = strsplit(SEPARATOR, msg)
	if key == KEY_VERREQ or key == KEY_VERRESP then
		BMTSyncing_VersionKeyHandler(msg, channel, sender)
	end
end

-- addon message key receiving handlers
function BMTSyncing_VersionKeyHandler(msg, channel, sender)
	local key, verNumStr = strsplit(SEPARATOR, msg)
	local verNum = tonumber(verNumStr)
	-- if received -> request, even if the version is lower
	if key == KEY_VERREQ then
		-- reply to the same channel we received the request from
		BMTSyncing_SendAddonMessage(KEY_VERRESP..SEPARATOR..BMTRevision, channel, sender)
	end
	-- always save highest revision
	BMTHighestRevision = verNum
	-- add the player to the list of players with addon
	BMTSyncing_AddOrUpdatePlayerToListWithAddon(sender, verNum, channel)
end

-- TODO: remove players from it when player goes offline or leaving party/raid
function BMTSyncing_AddOrUpdatePlayerToListWithAddon(playerName, version, channel)
	local numPlayers = #BMTPlayersWithAddon	
	for i=1, numPlayers do
		if BMTPlayersWithAddon[i]["name"] == playerName then
			-- update version
			BMTPlayersWithAddon[i]["ver"] = version
			local numChannels = #BMTPlayersWithAddon[i]["channels"]
			for j=1, numChannels do
				-- return, so we don't add this channel again
				if BMTPlayersWithAddon[i]["channels"][j] == channel then return end
			end
			-- if the function didn't return after the loop, we've found the player but he's communicating in a new channel
			BMTPlayersWithAddon[i]["channels"][numChannels+1] = channel
			return	-- so we don't add that player again
		end
	end
	-- if the function didn't return yet, this is a new player
	BMTPlayersWithAddon[numPlayers+1] = {}
	BMTPlayersWithAddon[numPlayers+1]["name"] = playerName
	BMTPlayersWithAddon[numPlayers+1]["ver"] = version
	BMTPlayersWithAddon[numPlayers+1]["channels"] = {}
	BMTPlayersWithAddon[numPlayers+1]["channels"][1] = channel
end

-- mainly to handle the spinning inset
function BMTSyncing_ChangeSyncingState(newState)
	BMTSyncingState = newState
	if BMTFrame:IsShown() then
		if BMTSyncingState == 0 then
			BMTFrameSpinningInset:Hide()
		else
			BMTFrameSpinningInset:Show()
		end
	end
	if BMTSyncingState == 1 then
		BMTFrameSpinningInset.Text:SetText(SYNCING_STATE_CHECK_VERSION_TEXT)
	end
end

-- asks for new version everywhere (called only upon logging in)
function BMTSyncing_AskForVersion_OnLogin()
	-- no need to check if window is shown here, as only happens upon logging in -> will be hidden
	-- guild
	BMTSyncing_ChangeSyncingState(SYNCING_STATE_CHECK_VERSION)
	BMTSyncing_SendAddonMessage(KEY_VERREQ..SEPARATOR..BMTRevision, "GUILD", nil)
	-- TODO: whisper
	-- TODO: party/raid
	BMTTaskManager_Schedule(SYNCING_TIMEOUT_VERSION, BMTSyncing_FinishSyncingVersion)
end

function BMTSyncing_FinishSyncingVersion()
	BMTSyncing_ChangeSyncingState(SYNCING_STATE_NONE)
	-- fade out only if we're not fading already
	if BMTHighestRevision > BMTRevision and BMTFadeState == 3 then
		BMTFadeState = -1
		BMTBottomText_FadingHandler()
	end
end

-- Spinning tiles part
function BMTSpinningInset_Initialize()
	-- if you change it here, change it in the BlackMarketTracker.xml
	local self = BMTFrameSpinningInset
	-- set rotations
	for i=0, 7 do
		_G[self:GetName().."Tile"..i]:SetRotation(BMTDegToRad(i*45));
		-- original size is 32x32, which is a little to big in this case
		_G[self:GetName().."Tile"..i]:SetWidth(SPINNING_TILE_SIZE);
		_G[self:GetName().."Tile"..i]:SetHeight(SPINNING_TILE_SIZE);
	end
	-- hide all tiles
	for i=0, 7 do
		_G[self:GetName().."Tile"..i]:SetAlpha(0);
	end
	-- fade in first tile
	BMTTileFadeIn(_G[self:GetName().."Tile0"], 0);
end

function BMTDegToRad(degrees)
	return ((PI/180)*degrees);
end

function BMTFadeInTile(frame, id)
	local fadeInfo={
    mode = "IN",
    timeToFade = SPINNING_TILE_FADE_IN_TIME,
    finishedFunc = BMTTileFadeIn,
    finishedArg1 = frame,
	finishedArg2 = id,
    }
    UIFrameFade(frame, fadeInfo);
end

function BMTTileFadeIn(self, id)
	local fadeInfo={
    mode = "OUT",
    timeToFade = SPINNING_TILE_FADE_OUT_TIME,
    finishedFunc = BMTTileFadeOut,
    finishedArg1 = self,
	finishedArg2 = id,
    }
    UIFrameFade(self, fadeInfo);

	-- fade in next too
	local nextId, nextTile;
	if (id>0) then
		nextId = id-1;
	else
		nextId = 7;
	end
	
	nextTile = _G[self:GetParent():GetName().."Tile"..nextId];
	BMTFadeInTile(nextTile, nextId);
end

-- for future use, if ever needed
function BMTTileFadeOut(self, id)	
end
